@@ -543,6 +543,88 @@ def reproject_multibandraster(composite_filepath, warped_filepath, target_epsg,
            ]
     run_subprocess(cmd, work_dir)
 
+def save_raster(array, destfile, driver, epsg, ulx, uly, pixel_size, data_type,
+                colortable=None, nodata_value=None):
+
+    Driver = gdal.GetDriverByName(driver)
+
+    if len(array.shape) ==2:
+        xsize = array.shape[1]
+        ysize = array.shape[0]
+        ds = Driver.Create(str(destfile), xsize, ysize, 1, data_type, options=["COMPRESS=DEFLATE", "TILED=YES"])
+        ds.SetGeoTransform((ulx, pixel_size, 0, uly, 0, -pixel_size))
+        proj = osr.SpatialReference()
+        proj.ImportFromEPSG(epsg)
+        ds.SetProjection(proj.ExportToWkt())
+        band = ds.GetRasterBand(1)
+        band.WriteArray(array, 0, 0)
+        # set nodata_value if specified
+        if nodata_value is not None:
+            band.SetNoDataValue(nodata_value)
+        # set Color table if specified
+        if colortable is not None:
+            clrs = gdal.ColorTable()
+            for value, rgb in colortable.items():
+                clrs.SetColorEntry(int(value), tuple(rgb))
+            band.SetRasterColorTable(clrs)
+
+    elif len(array.shape) == 3:
+        xsize = array.shape[2]
+        ysize = array.shape[1]
+        nband = array.shape[0]
+        ds = Driver.Create(str(destfile), xsize, ysize, nband, data_type, options=["COMPRESS=DEFLATE", "TILED=YES"])
+        ds.SetGeoTransform((ulx, pixel_size, 0, uly, 0, -pixel_size))
+        proj = osr.SpatialReference()
+        proj.ImportFromEPSG(epsg)
+        ds.SetProjection(proj.ExportToWkt())
+        for band_item in range(nband):
+            band = ds.GetRasterBand(band_item+1)
+            band.WriteArray(array[band_item,:,:], 0, 0)
+            # set nodata_value if specified
+            if nodata_value is not None:
+                band.SetNoDataValue(nodata_value)
+            # set Color table if specified
+            if colortable is not None:
+                clrs = gdal.ColorTable()
+                for value, rgb in colortable.items():
+                    clrs.SetColorEntry(int(value), tuple(rgb))
+                band.SetRasterColorTable(clrs)
+
+    band.FlushCache()
+    band, ds = None, None
+
+
+def save_raster_template(rasterfn, newRasterfn, array, data_type, nodata_value=None):
+
+    raster = gdal.Open(str(rasterfn))
+    geotransform = raster.GetGeoTransform()
+    originX = geotransform[0]
+    originY = geotransform[3]
+    pixelWidth = geotransform[1]
+    pixelHeight = geotransform[5]
+    cols = raster.RasterXSize
+    rows = raster.RasterYSize
+
+    if len(array.shape) == 2:
+        nband = 1
+    elif len(array.shape) == 3:
+        nband = array.shape[0]
+
+    driver = gdal.GetDriverByName('GTiff')
+    outRaster = driver.Create(str(newRasterfn), cols, rows, nband, data_type,
+                              ['COMPRESS=DEFLATE', 'TILED=YES', 'BIGTIFF=IF_NEEDED'])
+    outRaster.SetGeoTransform((originX, pixelWidth, 0, originY, 0, pixelHeight))
+    outRasterSRS = osr.SpatialReference()
+    outRasterSRS.ImportFromWkt(raster.GetProjectionRef())
+    outRaster.SetProjection(outRasterSRS.ExportToWkt())
+
+    for band_item in range(nband):
+        outband = outRaster.GetRasterBand(band_item + 1)
+        outband.WriteArray(array[band_item,:,:], 0, 0)
+        if nodata_value is not None:
+            outband.SetNoDataValue(nodata_value)
+    outband.FlushCache()
+
 
 #################################################################################################
 #################################################################################################
